/* Yet Another Forum.NET
 * Copyright (C) 2003-2005 Bjørnar Henden
 * Copyright (C) 2006-2013 Jaben Cargman
 * Copyright (C) 2014-2015 Ingo Herbote
 * http://www.yetanotherforum.net/
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at

 * http://www.apache.org/licenses/LICENSE-2.0

 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
namespace YAF.Controls
{
	#region Using

	using System;
	using System.Data;
	using System.Web.UI.WebControls;

	using YAF.Classes;
	using YAF.Classes.Data;
	using YAF.Core;
	using YAF.Core.Services;
	using YAF.Types;
	using YAF.Types.Extensions;
	using YAF.Types.Interfaces;
	using YAF.Types.Interfaces.Data;
	using YAF.Utils;

	#endregion

	/// <summary>
	/// The forum category list.
	/// </summary>
	public partial class ForumCategoryList : BaseUserControl
	{
		#region Methods

		/// <summary>
		/// Column count
		/// </summary>
		/// <returns>
		/// The column count.
		/// </returns>
		protected int ColumnCount()
		{
			int cnt = 5;

			if (this.Get<YafBoardSettings>().ShowModeratorList && this.Get<YafBoardSettings>().ShowModeratorListAsColumn)
			{
				cnt++;
			}

			return cnt;
		}

		/// <summary>
		/// The mark all_ click.
		/// </summary>
		/// <param name="sender">
		/// The sender.
		/// </param>
		/// <param name="e">
		/// The e.
		/// </param>
		protected void MarkAll_Click([NotNull] object sender, [NotNull] EventArgs e)
		{
			var markAll = (LinkButton)sender;

			object categoryId = null;

			int icategoryId;
			if (int.TryParse(markAll.CommandArgument, out icategoryId))
			{
				categoryId = icategoryId;
			}

			DataTable dt = LegacyDb.forum_listread(
				boardID: this.PageContext.PageBoardID,
				userID: this.PageContext.PageUserID,
				categoryID: categoryId,
				parentID: null,
				useStyledNicks: false,
				findLastRead: false);

			this.Get<IReadTrackCurrentUser>().SetForumRead(dt.AsEnumerable().Select(r => r["ForumID"].ToType<int>()));

			this.BindData();
		}

		/// <summary>
		/// The page_ load.
		/// </summary>
		/// <param name="sender">
		/// The sender.
		/// </param>
		/// <param name="e">
		/// The e.
		/// </param>
		protected void Page_Load([NotNull] object sender, [NotNull] EventArgs e)
		{
			this.BindData();
		}

		/// <summary>
		/// Bind Data
		/// </summary>
		private void BindData()
		{
			DataSet ds = this.Get<YafDbBroker>().BoardLayout(
					this.PageContext.PageBoardID, this.PageContext.PageUserID, this.PageContext.PageCategoryID, null);

			this.CategoryList.DataSource = ds.Tables["Category"];
			this.CategoryList.DataBind();
		}

		#endregion
	}
}